
import java.awt.Cursor;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.TooManyListenersException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.comm.*;
import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * ManagerInterface.java
 *
 * Created on May 1, 2010, 10:17:14 PM
 */

/**
 *
 * @author Daniel
 */
public class ManagerInterface extends javax.swing.JFrame 
                              implements SerialPortEventListener {

    private static int TIMEOUT = 30;
    private static int BUFFER_SIZE = 300; // Number of pw units * 2 (x and y)
    private static byte PW_GRID_SIZE_X = 32;
    private static byte PW_GRID_SIZE_Y = 24;

    // Messages
    private static byte MSG_SEND_PW = 'P';
    private static byte MSG_REQUEST_STATS = 'S';
    private static byte MSG_CLR_STATS = 'C';
    private static byte MSG_ACK = 'A';
    
    private InputStream is;
    private OutputStream os;
    private PortSelector selector;
    private SerialPort serialPort;
    private Cursor touchCursorOn;
    private Cursor touchCursorOff;
    private Timer pwTimer;
    private byte[] pwBuffer;
    private int pwPos;
    private boolean recordingPw;
    private boolean pwPressing;
    private boolean expectingStats;
     private boolean receivedAdmit;
     private boolean receivingStats;

    /** Creates new form ManagerInterface */
    public ManagerInterface() {
        initComponents();
        recordingPw = false;
        pwPressing = false;
        expectingStats = false;
        receivedAdmit = false;
        receivingStats = false;
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        BufferedImage imgOn = null;
        BufferedImage imgOff = null;
        try {
            imgOn = ImageIO.read(new File("finger_on.png"));
            imgOff = ImageIO.read(new File("finger_off.png"));
        } catch (IOException ex) {
            Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        touchCursorOn = toolkit.createCustomCursor(imgOn,
                new Point(imgOn.getWidth()/2, imgOn.getHeight()/2), "fingerOn");
        touchCursorOff = toolkit.createCustomCursor(imgOff,
                new Point(imgOff.getWidth()/2, imgOff.getHeight()/2), "fingerOff");
        pwPad.setCursor(touchCursorOff);

        pwBuffer = new byte[BUFFER_SIZE];
        pwPos = 0;
        /*
         * Saves the x and y positions when entering the password.
         */
        Action savePositionAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                ManagerInterface parent = ManagerInterface.this;
                if (parent.pwPos + 2 >= ManagerInterface.BUFFER_SIZE) { // Reached limit
                    pwTimer.stop();
                    sendPwButton.setEnabled(true);
                    pwStartStopButton.setText("Start");
                    pwStartStopButton.setSelected(false);
                    recordingPw = false;
                    return;
                } 
                Point pt = parent.pwPad.getMousePosition();
                int padSizeX = parent.pwPad.getWidth();
                int padSizeY = parent.pwPad.getHeight();

                if (pt != null && parent.pwPressing) {
                    parent.pwBuffer[parent.pwPos++] = normalizeX(pt.x, padSizeX);
                    parent.pwBuffer[parent.pwPos++] = normalizeY(pt.y, padSizeY);
                } else {
                    parent.pwBuffer[parent.pwPos++] = 0;
                    parent.pwBuffer[parent.pwPos++] = 0;
                }
            }
            // Methods for converting to grid coordinates
            private byte normalizeX(int x, int padSizeX) {
                byte gridSize = ManagerInterface.PW_GRID_SIZE_X;
                int step = padSizeX/gridSize;
                x = x/step;
                if (x < 0) return 0; 
                return x < gridSize ? (byte)x : gridSize;
            }
            private byte normalizeY(int y, int padSizeY) {
                byte gridSize = ManagerInterface.PW_GRID_SIZE_Y;
                int step = padSizeY/gridSize;
                y = gridSize - y/step;
                if (y < 0) return 0;
                return y < gridSize ? (byte)y : gridSize;
            }
        };
        pwTimer = new Timer(50, savePositionAction);
        selector = new PortSelector(this);
        selector.setVisible(true);

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        sendPwButton = new javax.swing.JButton();
        pwStartStopButton = new javax.swing.JToggleButton();
        jPanel4 = new javax.swing.JPanel();
        pwPad = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        admitCountLabel = new javax.swing.JLabel();
        rejectCountLabel = new javax.swing.JLabel();
        requestStatsButton = new javax.swing.JButton();
        clearStatsButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        logTextArea = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Security Manager");
        setAlwaysOnTop(true);

        sendPwButton.setText("Send");
        sendPwButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                sendPwButtonMouseClicked(evt);
            }
        });

        pwStartStopButton.setText("Start");
        pwStartStopButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                pwStartStopButtonMouseClicked(evt);
            }
        });

        jPanel4.setBackground(new java.awt.Color(204, 204, 204));

        pwPad.setBackground(new java.awt.Color(255, 255, 255));
        pwPad.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                pwPadMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                pwPadMouseReleased(evt);
            }
        });

        javax.swing.GroupLayout pwPadLayout = new javax.swing.GroupLayout(pwPad);
        pwPad.setLayout(pwPadLayout);
        pwPadLayout.setHorizontalGroup(
            pwPadLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 248, Short.MAX_VALUE)
        );
        pwPadLayout.setVerticalGroup(
            pwPadLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 165, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel4Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(pwPad, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(pwPad, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addGap(35, 35, 35)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 52, Short.MAX_VALUE)
                .addComponent(sendPwButton)
                .addGap(24, 24, 24))
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(132, 132, 132)
                .addComponent(pwStartStopButton)
                .addContainerGap(247, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addGap(42, 42, 42)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 41, Short.MAX_VALUE)
                .addComponent(pwStartStopButton)
                .addGap(28, 28, 28))
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(50, 50, 50)
                .addComponent(sendPwButton)
                .addContainerGap(248, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab("Access", jPanel1);

        jLabel1.setText("Admisison Count:");

        jLabel2.setText("Rejection Count:");

        admitCountLabel.setText("0");

        rejectCountLabel.setText("0");

        requestStatsButton.setText("Request Stats");
        requestStatsButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                requestStatsButtonMouseClicked(evt);
            }
        });

        clearStatsButton.setText("Clear Stats");
        clearStatsButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                clearStatsButtonMouseClicked(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGap(53, 53, 53)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addComponent(requestStatsButton)
                        .addGap(30, 30, 30)
                        .addComponent(clearStatsButton))
                    .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addGroup(jPanel2Layout.createSequentialGroup()
                            .addComponent(jLabel2)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(rejectCountLabel))
                        .addGroup(jPanel2Layout.createSequentialGroup()
                            .addComponent(jLabel1)
                            .addGap(100, 100, 100)
                            .addComponent(admitCountLabel))))
                .addContainerGap(167, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGap(56, 56, 56)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(admitCountLabel))
                .addGap(46, 46, 46)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(rejectCountLabel))
                .addGap(67, 67, 67)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(requestStatsButton)
                    .addComponent(clearStatsButton))
                .addContainerGap(101, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab("Stats", jPanel2);

        logTextArea.setColumns(20);
        logTextArea.setRows(5);
        jScrollPane1.setViewportView(logTextArea);

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGap(37, 37, 37)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 364, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(35, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 280, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(30, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab("Logger", jPanel3);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(19, 19, 19)
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 441, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(31, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(19, 19, 19)
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 349, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(19, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void sendPwButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_sendPwButtonMouseClicked
        try {
            os.write(MSG_SEND_PW);
            os.write((byte)(pwBuffer.length/2)); // Number of pw unit pairs
            os.write(pwBuffer);
        } catch (IOException ex) {
            Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
        }
        for (int i = 0; i < pwPos; i++) {
            System.out.println(pwBuffer[i]);
        }

}//GEN-LAST:event_sendPwButtonMouseClicked

    private void pwPadMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_pwPadMousePressed
        pwPad.setCursor(touchCursorOn);
        pwPressing = true;
}//GEN-LAST:event_pwPadMousePressed

    private void pwPadMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_pwPadMouseReleased
        pwPad.setCursor(touchCursorOff);
        pwPressing = false;
}//GEN-LAST:event_pwPadMouseReleased

    private void pwStartStopButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_pwStartStopButtonMouseClicked
        if (recordingPw) {
            pwTimer.stop();
            sendPwButton.setEnabled(true);
            pwStartStopButton.setText("Start");
            recordingPw = false;
        } else {
            // Start recording
            pwPos = 0;
            sendPwButton.setEnabled(false); // Can't send pw until finished
            pwStartStopButton.setText("Stop");
            recordingPw = true;
            pwTimer.start(); // Timer to start saving the pw coordinates
        }
}//GEN-LAST:event_pwStartStopButtonMouseClicked

    private void requestStatsButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_requestStatsButtonMouseClicked
        
        try {
            os.write(MSG_REQUEST_STATS);
            expectingStats = true;
        } catch (IOException ex) {
            Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_requestStatsButtonMouseClicked

    private void clearStatsButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_clearStatsButtonMouseClicked
        try {
            os.write(MSG_CLR_STATS);
            admitCountLabel.setText("0");
            rejectCountLabel.setText("0");
        } catch (IOException ex) {
            Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
        }

    }//GEN-LAST:event_clearStatsButtonMouseClicked

    
    public void serialEvent(SerialPortEvent spe) {
        //  byte[] buffer = new byte[BUFFER_SIZE];
        StringBuffer buffer = new StringBuffer();
        int numBytes = 0, num, rxElement;
        switch(spe.getEventType()) {
            case SerialPortEvent.DATA_AVAILABLE:
            try {
                if (is != null) {
                    while ((num = is.available()) > 0) {
                        rxElement = is.read();
                        if (expectingStats) {
                            expectingStats = false;
                            if (rxElement == MSG_ACK)
                                receivingStats = true;
                            else {
                                buffer.append((char)rxElement);
                                numBytes++;
                            }
                        } else if (receivingStats) {
                            if (receivedAdmit) {
                                rejectCountLabel.setText(Integer.toString(rxElement));
                                receivingStats = false;
                                receivedAdmit = false;
                            } else {
                                admitCountLabel.setText(Integer.toString(rxElement));
                                receivedAdmit = true;
                            }

                        } else {
                            
                            buffer.append((char)rxElement);
                            numBytes++;
                        }
                    }
                    logTextArea.insert(buffer.toString(), logTextArea.getText().length());
                }

            } catch (IOException ex) {
                Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
            }
                break;
            default:
                break;


        }
    }

    /*
     * Makes the main interface visible and initializes
     * the communication
     */
    public void initCommunication() {
        if (selector.getPortName() == null) {
            System.out.println("No port selected. Try again.\n");
            return;
        }
        this.initSerial();
        this.setVisible(true);
        selector.setVisible(false);
    }

     /*
     * Initializes the serial port and creates the input and output
     * streams
     */
    private void initSerial() {

        System.out.println("Trying to open " + selector.getPortID().getName() + "...");

        switch (selector.getPortID().getPortType()) {
            case CommPortIdentifier.PORT_SERIAL:

                try {
                    serialPort = (SerialPort)selector.getPortID().open("Dan Port", ManagerInterface.TIMEOUT*1000);
                } catch (PortInUseException ex) {
                    Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
                }
                try{
                     serialPort.addEventListener(this);
                } catch (TooManyListenersException ex) {
                    Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
                }
                serialPort.notifyOnDataAvailable(true);
                try {
                    serialPort.setSerialPortParams(selector.getBaudRate(),
                            SerialPort.DATABITS_8, SerialPort.STOPBITS_1,
                            SerialPort.PARITY_NONE);
                    serialPort.setFlowControlMode(serialPort.FLOWCONTROL_NONE);


                } catch (UnsupportedCommOperationException ex) {
                    Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
                }

                try{
                    is=serialPort.getInputStream();
                    os=serialPort.getOutputStream();

                } catch (IOException ex) {
                    Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
                }

                break;
            default:
                break;
        }
    }

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                try {
                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                } catch (ClassNotFoundException ex) {
                    Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
                } catch (InstantiationException ex) {
                    Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
                } catch (IllegalAccessException ex) {
                    Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
                } catch (UnsupportedLookAndFeelException ex) {
                    Logger.getLogger(ManagerInterface.class.getName()).log(Level.SEVERE, null, ex);
                }
                new ManagerInterface().setVisible(false);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel admitCountLabel;
    private javax.swing.JButton clearStatsButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextArea logTextArea;
    private javax.swing.JPanel pwPad;
    private javax.swing.JToggleButton pwStartStopButton;
    private javax.swing.JLabel rejectCountLabel;
    private javax.swing.JButton requestStatsButton;
    private javax.swing.JButton sendPwButton;
    // End of variables declaration//GEN-END:variables


}
